/*	cccontroller.h
	Controller class
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		cccontroller.h
 *	@brief		<code>ccController</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
/* Ah more of the platform specific code... */

#ifndef __CCCONTROLLER_H
#define __CCCONTROLLER_H

#include <ccsys.h>

namespace crabclaw	{
	/**
	 *	Controller/joystick class.
	 *	This class provides a common method to poll controllers
	 *	and joysticks.
	 *	@since	0.9.0
	 */
	class ccController	{
		private:
			int32 *axis;
			uint32 buttons;
			uint8 num_buttons;
			uint8 num_axis;
			int cont_num;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccController</code> object
			 *	for a specified device number.
			 *	@param	num		The device number.
			 *	@since			0.9.0
			 */
			ccController(int num);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this
			 *	<code>ccController</code>
			 */
			~ccController();
			
			/**
			 *	Get the button status.
			 *	Get the complete button status bitfield.
			 *	@return			Status bitfield.
			 *	@since			0.9.0
			 */
			uint32 getButtons()			{	return buttons;					}
			
			/**
			 *	Get the status of an axis.
			 *	Get the status of a numbered axis on
			 *	this <code>ccController</code>.
			 *	@param	num		Axis number.
			 *	@return			Axis status.
			 *	@since			0.9.0
			 */
			int32 getAxis(int num)		{	return axis[num];				}
			
			/**
			 *	Get the device number.
			 *	Get the device number of this
			 *	<code>ccController</code>.
			 *	@return			Device number.
			 *	@since			0.9.0
			 */
			uint32 getNumber()			{	return cont_num;				}
			
			/**
			 *	Get the number of axis.
			 *	Get the number of axis of this
			 *	<code>ccController</code>.
			 *	@return			Number of axis.
			 *	@since			0.9.0
			 */
			uint8 getAxisCount()		{	return num_axis;				}
			
			/**
			 *	Get the number of buttons.
			 *	Get the number of buttons on this
			 *	<code>ccController</code>.
			 *	@return			Number of buttons.
			 *	@since			0.9.0
			 */
			uint8 getButtonCount()		{	return num_buttons;				}
			
			/**
			 *	Determine if a button is pressed.
			 *	Determines if a given button is pressed.
			 *	@return			true if the button is pressed.
			 *	@since			0.9.0
			 */
			bool isPressed(int button)	{	return buttons & (1 << button);	}
			
			/**
			 *	Update available info about this controller.
			 *	Update bitfields and axis status for this
			 *	<code>ccController</code>.
			 *	@since			0.9.0
			 */
			void poll();
	};
}

#endif
