/*	ccfont.h
	Font classes
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccfont.h
 *	@brief		<code>ccTTFFont</code> and related structures and classes.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
#ifndef __CCFONT_H
#define __CCFONT_H

#include <ccvideo.h>
#include <ccvertex.h>
#include <cctexture.h>
#include <ccdrawable.h>
#include <ft2build.h>
#include FT_FREETYPE_H

namespace crabclaw	{
	/**
	 *	TTF font glyph structure.
	 *	A structure holding all the attributes of a glyph
	 *	in a TTF font.
	 *	@since	0.9.0
	 */
	typedef struct	{
		/**
		 *	Glyph height.
		 */
		int h;
		
		/**
		 *	Glyph width.
		 */
		int w;
		
		/**
		 *	U coordinate of the last column of the actual glyph.
		 */
		float u;
		
		/**
		 *	V coordinate of the last row of the glyph
		 */
		float v;
		
		/**
		 *	How far below the baseline the glyph extends.
		 */
		float below;
		
		/**
		 *	How far to advance the cursor after the glyph.
		 */
		float adv;
		
		/**
		 *	How far to advance the cursor before the glyph.
		 */
		int advb;
	} ttf_glyph_t;
	
	/**
	 *	TTF font class.
	 *	This class wraps around the functionality provided by
	 *	<a href="http://www.freetype.org/" target="_new">FreeType</a> to
	 *	provide an easy-to-use interface for TTF fonts.
	 *	@since	0.9.0
	 */	
	class ccTTFFont	{
		private:
			ccTexture **txr;
			ttf_glyph_t *glyph;
			float height;
			bool lib_initted;
			FT_Library lib;
			
		public:
			/**
			 *	Constructor.
			 *	Create a new <code>ccTTFFont</code> object from a
			 *	.ttf file.
			 *	@param	fn		The file of the font.
			 *	@param	h		The height to render textures at.
			 *	@since			0.9.0
			 */
			ccTTFFont(char *fn, float h);
			
			/**
			 *	Destructor.
			 *	Cleans up all memory associated with this
			 *	<code>ccTTFFont</code>.
			 */
			~ccTTFFont();
			
			/**
			 *	Draw a string of text.
			 *	Draw a string of text with this <code>ccTTFFont</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@param	c		The string to draw.
			 *	@param	x		The horizontal position of the start of
			 *					the text, in pixels.
			 *	@param	y		The vertical position of the baseline
			 *					of the text string.
			 *	@param	height	The height to draw the string at.
			 *	@param	argb	The color to draw the string.
			 *	@since			0.9.0
			 *	@see			ccTextString
			 */
			void drawString(ccScreen *scr, const char *c, int x, int y, float height, uint32 argb = 0xFFFFFFFF);
			
			/**
			 *	Draw a formatted text string.
			 *	Draw a formatted text string with this <code>ccTTFFont</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@param	x		The horizontal position of the test, in pixels.
			 *	@param	y		The vertical position of the baseline of the
			 *					text string, in pixels.
			 *	@param	height	The height to draw the string at.
			 *	@param	argb	The color to draw the string.
			 *	@param	fmt		The format string.
			 *	@param	...		Items to format.
			 *	@since			0.9.0
			 */
			void printf(ccScreen *scr, int x, int y, float height, uint32 argb, const char *fmt, ...);	
	};
	
	/**
	 *	Drawable strings of text.
	 *	This class is designed to be used in conjunction with
	 *	the <code>ccSceneManager</code> class to facilitate the drawing of
	 *	a string of text in a scene.
	 *	@see	ccSceneManager
	 *	@see	ccDrawable
	 *	@since	0.9.0
	 */
	class ccTextString : public ccDrawable	{
		private:
			char *str;
			ccTTFFont *f;
			int x;
			int y;
			float height;
			float width;
			uint32 argb;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccTextString</code>.
			 *	@param	font	The <code>ccTTFFont</code> to use in this string.
			 *	@param	string	The string to be drawn.
			 *	@param	height	The height to draw at.
			 *	@param	color	The color to draw the text at.
			 *	@since			0.9.0
			 */
			ccTextString(ccTTFFont *font, char *string, float height, uint32 color = 0xFFFFFFFF);
			
			/**
			 *	Destructor.
			 *	Cleans up memory associated with this <code>ccTextString</code>,
			 *	but does not destroy the font.
			 *	@since			0.9.0
			 */
			~ccTextString();
			
			/**
			 *	Set the position to draw at.
			 *	Set the position, in pixels, to draw the string at.
			 *	@param	xp		The horizontal position of the text.
			 *	@param	yp		The vertical position of the baseline of the text.
			 *	@since			0.9.0
			 */
			void setPosition(int xp, int yp);
			
			/**
			 *	Draw the text.
			 *	Draw the text to the specified <code>ccScreen</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			void draw(ccScreen *scr);
			
			/**
			 *	Get the list to draw the text to.
			 *	Get the list that this <code>ccTextString</code> should
			 *	be drawn in.
			 *	@return			ccScreen::LIST_TRANSLUCENT
			 *	@since			0.9.0
			 */
			int getList()	{	return ccScreen::LIST_TRANSLUCENT;	}
	};
}

#endif
