/*	ccline.h
	Lines
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 *	@file		ccline.h
 *	@brief		Declaration of the <code>ccLine</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCLINE_H
#define __CCLINE_H

#include <ccsys.h>
#include <ccvideo.h>
#include <ccdrawable.h>

namespace crabclaw	{
	/**
	 *	Drawable Lines.
	 *	This class is designed to be used in conjunction with
	 *	the <code>ccSceneManager</code> class to facilitate the drawing of
	 *	lines in a scene.
	 *	@see	ccSceneManager
	 *	@see	ccDrawable
	 *	@since	0.9.0
	 */
	class ccLine : public ccDrawable	{
		private:
			int list;
			int xp1;
			int xp2;
			int yp1;
			int yp2;
			int zp;
			int thickness;
			uint32 col;
			int t;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccLine</code> object.
			 *	@param lst		The primitive list that this <code>ccLine</code> should be drawn in.
			 *	@param x1		The X position of the first endpoint of this <code>ccLine</code>.
			 *	@param y1		The Y position of the first endpoint of this <code>ccLine</code>.
			 *	@param x2		The X position of the second endpoint of this <code>ccLine</code>.
			 *	@param y2		The Y position of the second endpoint of this <code>ccLine</code>.
			 *	@param z		The Z position of this <code>ccLine</code>.
			 *	@param thick	The thickness of this <code>ccLine</code>.
			 *	@param c		The color of this <code>ccLine</code>.
			 *	@param	ty		The type of line.
			 *	@since			0.9.0
			 */
			 ccLine(int lst, int x1, int y1, int x2, int y2, int z, int thick, uint32 c, int ty);
			
			/**
			 *	Destructor.
			 *	Cleans up any used memory when the line is no longer needed
 			 *	@since			0.9.0
			 */
			~ccLine();
			
			/**
			 *	Creates a new <code>ccLine</code> that is horizontal.
			 *	@param lst		The primitive list that this <code>ccLine</code> should be drawn in.
			 *	@param x1		The X position of one endpoint of this <code>ccLine</code>.
			 *	@param x2		The X position of one endpoint of this <code>ccLine</code>.
			 *	@param y		The Y position of this <code>ccLine</code>.
			 *	@param z		The Z position of this <code>ccLine</code>.
			 *	@param thick	The thickness of this <code>ccLine</code>.
			 *	@param c		The color of this <code>ccLine</code>.
			 *	@return			A new <code>ccLine</code> object.
			 *	@since			0.9.0
			 */
			static ccLine *createHorizontalLine(int lst, int x1, int x2, int y, int z, int thick, uint32 c);
			/**
			 *	Creates a new <code>ccLine</code> that is vertical.
			 *	@param lst		The primitive list that this <code>ccLine</code> should be drawn in.
			 *	@param x		The X position of this <code>ccLine</code>.
			 *	@param y1		The Y position of one endpoint of this <code>ccLine</code>.
			 *	@param y2		The Y position of one endpoint of this <code>ccLine</code>.
			 *	@param z		The Z position of this <code>ccLine</code>.
			 *	@param thick	The thickness of this <code>ccLine</code>.
			 *	@param c		The color of this <code>ccLine</code>.
			 *	@return			A new <code>ccLine</code> object.
			 *	@since			0.9.0
			 */
			static ccLine *createVerticalLine(int lst, int x, int y1, int y2, int z, int thick, uint32 c);
			/**
			 *	Creates a new <code>ccLine</code> that is diagonal.
			 *	@param lst		The primitive list that this <code>ccLine</code> should be drawn in.
			 *	@param x1		The X position of the first endpoint of this <code>ccLine</code>.
			 *	@param y1		The Y position of the first endpoint of this <code>ccLine</code>.
			 *	@param x2		The X position of the second endpoint of this <code>ccLine</code>.
			 *	@param y2		The Y position of the second endpoint of this <code>ccLine</code>.
			 *	@param z		The Z position of this <code>ccLine</code>.
			 *	@param thick	The thickness of this <code>ccLine</code>.
			 *	@param c		The color of this <code>ccLine</code>.
			 *	@return			A new <code>ccLine</code> object.
			 *	@since			0.9.0
			 */
			static ccLine *createDiagonalLine(int lst, int x1, int y1, int x2, int y2, int z, int thick, uint32 c);
			
			/**
			 *	Draws this <code>ccLine</code> to the screen.
			 *	Draws this <code>ccLine</code> to the screen, under the assumption that
			 *	the correct polygon list has already been opened (For instance by a
			 *	<code>ccSceneManager</code> object).
			 *	@param scr		The <code>ccScreen</code> object to draw this <code>ccLine</code> to.
			 *	@since			0.9.0
			 */			
			void draw(ccScreen *scr);
			
			/**
			 *	Gives the primitive list that this <code>ccLine</code> should be drawn in.
			 *	@return			The list type.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			int getList()					{	return list;	}
	};
}

#endif
