/*	cclist.h
	Linked Lists
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		cclist.h
 *	@brief		Templated linked list class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCLIST_H
#define __CCLIST_H

#include <ccsys.h>

namespace crabclaw	{
	
	/**
	 *	Templated list node class.
	 *	This class provides a templated list node for use in
	 *	<code>ccList</code>.
	 *	@since	0.9.0
	 */
	template<class T>
	class ccListNode	{
		private:
			ccListNode<T> *next;
			ccListNode<T> *prev;
			T *data;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccListNode</code> object.
			 *	@param	p		The previous node, if any.
			 *	@param	n		The next node, if any.
			 *	@param	dat		The payload data.
			 *	@since			0.9.0
			 */
			ccListNode(ccListNode<T> *p, ccListNode<T> *n, T *dat);
			
			/**
			 *	Destructor.
			 *	Cleans up memory associated with this
			 *	<code>ccListNode</code>. Deletes the data
			 *	associated with it also.
			 *	@since			0.9.0
			 */
			~ccListNode();
			
			/**
			 *	Get the data in this <code>ccListNode</code>.
			 *	Get the data that this <code>ccListNode</code> contains.
			 *	@return			The data.
			 *	@since			0.9.0
			 */
			T *getData();
			
			/**
			 *	Set the data in this <code>ccListNode</code>.
			 *	Delete any old data, and set the data.
			 *	@param	d		The new data.
			 *	@since			0.9.0
			 */
			void setData(T *d);
			
			/**
			 *	Get the next <code>ccListNode</code>.
			 *	Get the <code>ccListNode</code> that follows this one
			 *	in the current <code>ccList</code>.
			 *	@return			The next node.
			 *	@since			0.9.0
			 */
			ccListNode<T> *getNext();
			
			/**
			 *	Get the previous <code>ccListNode</code>.
			 *	Get the <code>ccListNode</code> that precedes this one
			 *	in the current <code>ccList</code>.
			 *	@return			The previous node.
			 *	@since			0.9.0
			 */
			ccListNode<T> *getPrev();
			
			/**
			 *	Set the next <code>ccListNode</code>.
			 *	Set the <code>ccListNode</code> that follows this one
			 *	in the current <code>ccList</code>.
			 *	@param	n		The next node.
			 *	@since			0.9.0
			 */
			void setNext(ccListNode<T> *n);
			
			/**
			 *	Set the previous <code>ccListNode</code>.
			 *	Set the <code>ccListNode</code> that precedes this one
			 *	in the current <code>ccList</code>.
			 *	@param	p		The previous node.
			 *	@since			0.9.0
			 */
			void setPrev(ccListNode<T> *p);
	};
	
	/**
	 *	Templated linked list class.
	 *	This class provides a templated linked list.
	 *	@since	0.9.0
	 */
	template<class T>
	class ccList	{
		private:
			int len;
			ccListNode<T> *first;
			ccListNode<T> *last;
			
		public:
			/**
			 *	Constructor.
			 *	Creates an empty <code>ccList</code>
			 *	@since			0.9.0
			 */
			ccList();
			
			/**
			 *	Destructor.
			 *	Cleans up memory associated with this <code>ccList</code>,
			 *	and deletes all the <code>ccListNode</code> objects in it.
			 *	@since			0.9.0
			 */
			~ccList();
			
			/**
			 *	Get an object by number.
			 *	Get the object at position n.
			 *	@param	n		The object to get.
			 *	@return			The object.
			 *	@since			0.9.0
			 */
			T *get(int n);
			
			/**
			 *	Add an object to this <code>ccList</code>.
			 *	Add an object to the end of this linked-list.
			 *	@param	o		The object to add.
			 *	@since			0.9.0
			 */
			void add(T *o);
			
			/**
			 *	Remove an object from this <code>ccList</code>.
			 *	@param	n		The position to remove.
			 *	@since			0.9.0
			 */
			void remove(int n);
			
			/**
			 *	Get the size of this <code>ccList</code>.
			 *	Get the number of nodes in this <code>ccList</code>.
			 *	@return			The number of nodes.
			 *	@since			0.9.0
			 */
			int size()	const	{	return len;		}
			
			/**
			 *	Get the first node of this <code>ccList</code>.
			 *	Get the node at the front of this <code>ccList</code>.
			 *	@return			The front node.
			 *	@since			0.9.0
			 */
			ccListNode<T> *getFirstNode();
	};

	template<class T>
	ccListNode<T>::ccListNode(ccListNode<T> *p, ccListNode<T> *n, T *dat)	{
		prev = p;
		next = n;
		data = dat;
	}
	
	template<class T>
	ccListNode<T>::~ccListNode()	{
		delete data;
	}
	
	template<class T>
	T *ccListNode<T>::getData()	{
		return data;
	}
	
	template<class T>
	void ccListNode<T>::setData(T *d)	{
		delete data;
		data = d;
	}
	
	template<class T>
	ccListNode<T> *ccListNode<T>::getNext()	{
		return next;
	}
	
	template<class T>
	ccListNode<T> *ccListNode<T>::getPrev()	{
		return prev;
	}
	
	template<class T>
	void ccListNode<T>::setNext(ccListNode<T> *n)	{
		next = n;
	}
	
	template<class T>
	void ccListNode<T>::setPrev(ccListNode<T> *p)	{
		prev = p;
	}
	
	template<class T>
	ccList<T>::ccList()	{
		first = last = NULL;
		len = 0;
	}
	
	template<class T>
	ccList<T>::~ccList()	{
		ccListNode<T> *n = first;
		ccListNode<T> *tmp;
		while(n != NULL)	{
			tmp = n;
			n = n->getNext();
			delete tmp;
		}
	}
	
	template<class T>
	T *ccList<T>::get(int n)	{
		int i = 0;
		ccListNode<T> *cur = first;
		
		while(i < n)	{
			i++;
			cur = cur->getNext();
		}
		
		return cur->getData();
	}
	
	template<class T>
	void ccList<T>::remove(int n)	{
		int i = 0;
		ccListNode<T> *cur = first;
		
		while(i < n)	{
			i++;
			cur = cur->getNext();
		}
		
		cur->getPrev()->setNext(cur->getNext());
		cur->getNext()->setPrev(cur->getPrev());
		delete cur;
		len--;
	}
	
	template<class T>
	void ccList<T>::add(T *o)	{
		ccListNode<T> *t = new ccListNode<T>(last, NULL, o);
		last->setNext(t);
		last = t;
		if(first == NULL)	{
			first = t;
		}
		len++;
	}
	
	template<class T>
	ccListNode<T> *ccList<T>::getFirstNode()	{
		return first;
	}
}


#endif
