/*	ccmap.h
	.ljm map class
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccmap.h
 *	@brief		<code>ccLJMMap</code> and related structures.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
#ifndef __CCMAP_H
#define __CCMAP_H
	
#include <ccvideo.h>
#include <ccvertex.h>
#include <ccsprite.h>
#include <ccpolygon.h>
#include <ccdrawable.h>

namespace crabclaw	{
	/**
	 *	A tile on a LJM map.
	 *	Structure that contains all the user-specified
	 *	attributes of each tile.
	 *	@since		0.9.0
	 */
	typedef struct	{
		/**
		 *	32-bit user data.
		 */
		uint32 user1;
		
		/**
		 *	16-bit user data.
		 */
		uint16 user2, user3;
		
		/**
		 *	8-bit user data.
		 */
		uint8 user4, user5, user6;
	} ljmTile;
	
	/**
	 *	An event on a LJM map.
	 *	Structure that contains the data used in event
	 *	processing in a LJM map.
	 *	@since		0.9.0
	 */	
	typedef struct	{
		/**
		 *	Event position.
		 */
		uint32 evx, evy;
		
		/**
		 *	Event type.
		 */
		uint8 evtype;
		
		/**
		 *	Event user data (32-bit).
		 */
		uint32 evd1, evd2, evd3;
	} ljmEvent;
	
	/**
	 *	An animation on a LJM map.
	 *	Structure that contains the data used in
	 *	animating tiles on a LJM map.
	 *	@since		0.9.0
	 */	
	typedef struct	{
		/**
		 *	Frame count of the animation.
		 */
		uint32 count;
		
		/**
		 *	Frame sequence.
		 */
		uint32 * seq;
		
		/**
		 *	Delay between frames.
		 */
		uint8 delay;
		
		/**
		 *	Time the animation was last incremented, in miliseconds.
		 */
		uint64 sttime;
		
		/**
		 *	Current frame.
		 */
		uint32 current;
	} ljmAnim;
	
	/**
	 *	Drawable maps.
	 *	This class is designed to be used in conjunction with
	 *	the <code>ccSceneManager</code> class to facilitate the drawing of
	 *	a map in a scene.
	 *	@see	ccSceneManager
	 *	@see	ccDrawable
	 *	@since	0.9.0
	 */
	class ccLJMMap : public ccDrawable	{
		private:
			ccTexture **tiles_txr;
			uint32 **tile_map;
			uint8 dim;
			uint32 map_x;
			uint32 map_y;
			uint32 xoff;
			uint32 yoff;
			uint32 version;
			uint32 tcount;
			uint32 ecount;
			uint32 acount;
			ljmTile *tile;
			ljmEvent *event;
			ljmAnim *anim;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccLJMMap</code> object.
			 *	@param	fn		The file to create this 
			 *					<code>ccLJMMap</code> from.
			 *	@param	scale	The scale factor to modify dimensions in
			 *					this <code>ccLJMMap</code> with.
			 *	@since			0.9.0
			 */
			ccLJMMap(const char *fn, int scale = 1);
			
			/**
			 *	Destructor.
			 *	Cleans up all memory associated with this 
			 *	<code>ccLJMMap</code>.
			 *	@since			0.9.0
			 */
			~ccLJMMap();
			
			/**
			 *	Get the tile at a position on this <code>ccLJMMap</code>.
			 *	Get the tile at a position (in blocks) on this
			 *	<code>ccLJMMap</code>.
			 *	@param	x		The column of this <code>ccLJMMap</code>
			 *					to get.
			 *	@param	y		The row of this <code>ccLJMMap</code> to
			 *					get.
			 *	@return			The <code>ljmTile</code> structure of the
			 *					tile at the specified position.
			 *	@since			0.9.0
			 */					
			ljmTile *getBlock(uint32 x, uint32 y);
			
			/**
			 *	Get a tile of this <code>ccLJMMap</code>.
			 *	Get the tile based on its tile number on this
			 *	<code>ccLJMMap</code>.
			 *	@param	blknum	The tile of this <code>ccLJMMap</code>
			 *					to get.
			 *	@return			The <code>ljmTile</code> structure of the
			 *					specified tile.
			 *	@since			0.9.0
			 */	
			ljmTile *getBlock(uint32 blknum);
			
			/**
			 *	Get the tile at a position on this <code>ccLJMMap</code>.
			 *	Get the tile at a position (in pixels) on this
			 *	<code>ccLJMMap</code>.
			 *	@param	x		The column of this <code>ccLJMMap</code>
			 *					to get.
			 *	@param	y		The row of this <code>ccLJMMap</code> to
			 *					get.
			 *	@return			The <code>ljmTile</code> structure of the
			 *					tile at the specified position.
			 *	@since			0.9.0
			 */	
			ljmTile *getBlockPixels(uint32 x, uint32 y);
			
			/**
			 *	Set the X offset of this <code>ccLJMMap</code>.
			 *	Set the first column of tiles displayed on this
			 *	<code>ccLJMMap</code>.
			 *	@param	off		The first column of tiles to show.
			 *	@return			0 on success, -1 on failure
			 *	@since			0.9.0
			 */
			int setXOffset(uint32 off);
			
			/**
			 *	Set the Y offset of this <code>ccLJMMap</code>.
			 *	Set the first row of tiles displayed on this
			 *	<code>ccLJMMap</code>.
			 *	@param	off		The first row of tiles to show.
			 *	@return			0 on success, -1 on failure
			 *	@since			0.9.0
			 */
			int setYOffset(uint32 off);
			
			/**
			 *	Set the offsets of this <code>ccLJMMap</code>.
			 *	Set the first column and row of tiles displayed on this
			 *	<code>ccLJMMap</code>.
			 *	@param	xpos	The first column of tiles to show.
			 *	@param	ypos	The first row of tiles to show.
			 *	@return			0 on success, -1 on failure
			 *	@since			0.9.0
			 */
			int setOffsets(uint32 xpos, uint32 ypos);
			
			/**
			 *	Centers this <code>ccLJMMap</code> on a given tile.
			 *	Centers this <code>ccLJMMap</code> so that the tile
			 *	at (x, y) is at the center of the screen.
			 *	@param	x		The X coordinate to center on.
			 *	@param	y		The Y coordinate to center on.
			 *	@param	scx		The horizontal resolution of the display.
			 *	@param	scy		The vertical resolution of the display.
			 *	@return			0 on success, -1 on failure
			 *	@since			0.9.0
			 */
			int center(uint32 x, uint32 y, uint32 scx, uint32 scy);
			
			/**
			 *	Draw this <code>ccLJMMap</code>.
			 *	Draw this <code>ccLJMMap</code> on a given
			 *	<code>ccScreen</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			void draw(ccScreen *scr);
			
			/**
			 *	Get the primitive list of this <code>ccLJMMap</code>.
			 *	Get the list that this <code>ccLJMMap</code> should
			 *	be drawn in.
			 *	@return			ccScreen::LIST_OPAQUE
			 *	@since			0.9.0
			 */
			int getList()		{	return ccScreen::LIST_OPAQUE;	}
			
			/**
			 *	Increment the animations of this <code>ccLJMMap</code>.
			 *	Check if any animations need their frame changed, and
			 *	change them.
			 *	@since			0.9.0
			 */
			void incrementAnims();
			
			/**
			 *	Get an event at a given location.
			 *	Get any event at a given coordinate (in blocks) on this
			 *	<code>ccLJMMap</code>.
			 *	@param	x		The column to get an event from.
			 *	@param	y		The row to get an event from.
			 *	@return			The event at (x, y) or NULL if no
			 *					event is at the specified location.
			 *	@since			0.9.0
			 */
			ljmEvent *getEvent(uint32 x, uint32 y);
			
			/**
			 *	Get an event of a given type at a given location.
			 *	Get any event at a given coordinate (in blocks) on this
			 *	<code>ccLJMMap</code> that is of a given type.
			 *	@param	x		The column to get an event from.
			 *	@param	y		The row to get an event from.
			 *	@param	type	The type of event to get.
			 *	@return			The event at (x, y) or NULL if no
			 *					event of the given type exists at
			 *					the specified location.
			 *	@since			0.9.0
			 */
			ljmEvent *getEvent(uint32 x, uint32 y, uint8 type);
	};
}

#endif
