/*	ccmenu.h
	Generic Menus
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccmenu.h
 *	@brief		Menus and related classes.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
#ifndef __CCMENU_H
#define __CCMENU_H

#include <ccsys.h>
#include <cclist.h>
#include <ccvideo.h>
#include <ccpolygon.h>
#include <ccdrawable.h>

namespace crabclaw	{
	
	/**
	 *	Menu item class.
	 *	Menu items are used in <code>ccMenu</code>s.
	 *	@since	0.9.0
	 */	
	class ccMenuItem	{
		private:
			ccDrawable *o_unsel;
			ccDrawable *o_sel;
			bool fdraw;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccMenuItem</code>.
			 *	@param	unsel	The item to be shown when it is not selected.
			 *	@param	sel		The item to be shown when it is selected.
			 *	@param	freed	Whether or not to free the drawables in the destructor.
			 *	@since			0.9.0
			 */
			ccMenuItem(ccDrawable *unsel, ccDrawable *sel, bool freed = false);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this <code>ccMenuItem</code>.
			 *	@since			0.9.0
			 */
			~ccMenuItem();
			
			/**
			 *	Get the unselected item.
			 *	Get what item should be drawn when it is unselected.
			 *	@return			Unselected <code>ccDrawable</code>.
			 *	@since			0.9.0
			 */
			ccDrawable *getUnsel();
			
			/**
			 *	Get the selected item.
			 *	Get what item should be drawn when it is selected.
			 *	@return			Selected <code>ccDrawable</code>.
			 *	@since			0.9.0
			 */
			ccDrawable *getSel();
	};
	
	/**
	 *	Menu class.
	 *	A generic menu.
	 *	@since	0.9.0
	 *	@see	ccSceneManager
	 *	@see	ccSelectorMenu
	 */
	class ccMenu	{
		private:
			ccList<ccMenuItem> *it_list;
			ccList<ccDrawable> *o_list;
			int selected;
			void (*event_hnd)(int);
			ccScreen *scr;
			bool stopped;
			
		protected:
			/**
			 *	Initialize this <code>ccMenu</code>.
			 *	Provided as a convenience to subclasses.
			 *	@param	s		The <code>ccScreen</code> to draw to.
			 *	@param	initsel	Number of item that should be selected, initially.
			 *	@param	eh		Event handler callback, called each frame, takes
			 *					one argument representing the selected item.
			 *	@since			0.9.0
			 */
			void init(ccScreen *s, int initsel, void (*eh)(int));
			
			/**
			 *	Deinitialize this <code>ccMenu</code>.
			 *	Provided as a convenience to subclasses.
			 *	@since			0.9.0
			 */
			void deinit();
		
		public:
			/**
			 *	Constructor.
			 *	Calls <code>init()</code>.
			 *	@param	s		The <code>ccScreen</code> to draw to.
			 *	@param	initsel	Number of item that should be selected, initially.
			 *	@param	eh		Event handler callback, called each frame.
			 *	@since			0.9.0
			 */
			ccMenu(ccScreen *s, int initsel, void (*eh)(int));
			
			/**
			 *	Destructor.
			 *	Calls <code>deinit()</code>.
			 */
			virtual ~ccMenu();
			
			/**
			 *	Start this <code>ccMenu</code>.
			 *	Start this <code>ccMenu</code>'s control of the display.
			 *	This function will not return until <code>stop()</code> is called.
			 *	@since			0.9.0
			 */
			virtual void run();
			
			/**
			 *	Stop this <code>ccMenu</code>.
			 *	Break this <code>ccMenu</code>'s control of the display.
			 *	@since			0.9.0
			 */
			void stop();
			
			/**
			 *	Add an item to this <code>ccMenu</code>.
			 *	Add a <code>ccMenuItem</code> to this <code>ccMenu</code>.
			 *	@param	mi		The <code>ccMenuItem</code> to add.
			 *	@since			0.9.0
			 */
			void addItem(ccMenuItem *mi);
			
			/**
			 *	Add an object to this <code>ccMenu</code>.
			 *	Add a background object to this <code>ccMenu</code>.
			 *	@param	dr		The <code>ccDrawable</code> to add.
			 *	@since			0.9.0
			 */
			void addObject(ccDrawable *dr);
			
			/**
			 *	Set the selected item.
			 *	Set which item is selected in this <code>ccMenu</code>.
			 *	@param	sel		The item to select.
			 *	@since			0.9.0
			 */
			void setSelected(int sel);
	};
	
	/**
	 *	Selector-based menu class.
	 *	A generic menu that has a polygonal selector.
	 *	@since	0.9.0
	 *	@see	ccSceneManager
	 *	@see	ccMenu
	 */	
	class ccSelectorMenu	{
		private:
			ccList<ccDrawable> *o_list;
			int sel;
			int xoff;
			int yoff;
			int dx;
			int dy;
			int oldsel;
			
			void (*event_hnd)(int, void *);
			void *p;
			ccScreen *scr;
			bool stopped;			
			ccMovingPoly *selector;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccSelectorMenu</code>.
			 *	@param	s		The <code>ccScreen</code> to draw to.
			 *	@param	sel		The polygonal selector.
			 *	@param	initsel	The initially selected item.
			 *	@param	xo		The horizontal position the selector should
			 *					be placed at to select the first item.
			 *	@param	yo		The vertical position the selector should
			 *					be placed at to select the first item.
			 *	@param	x_c		Change in X per item.
			 *	@param	y_c		Change in Y per item.
			 *	@param	eh		Event handler callback, called each frame, takes
			 *					two arguments representing the selected item, and
			 *					a user specified parameter.
			 *	@param	param	Parameter to callback.
			 *	@since			0.9.0
			 */
			ccSelectorMenu(ccScreen *s, ccMovingPoly *sel, int initsel, int xo, int yo, int x_c, int y_c, void (*eh)(int, void*), void *param);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this <code>ccSelectorMenu</code>.
			 *	@since			0.9.0
			 */
			~ccSelectorMenu();
			
			/**
			 *	Start this <code>ccSelectorMenu</code>.
			 *	Start this <code>ccSelectorMenu</code>'s control of the display.
			 *	This function will not return until <code>stop()</code> is called.
			 *	@since			0.9.0
			 */
			void run();
			
			/**
			 *	Stop this <code>ccSelectorMenu</code>.
			 *	Break this <code>ccSelectorMenu</code>'s control of the display.
			 *	@since			0.9.0
			 */
			void stop();
			
			/**
			 *	Add an object to this <code>ccSelectorMenu</code>.
			 *	Add any <code>ccDrawable</code> to the list of items to be drawn.
			 *	@param	dr		The object to add.
			 *	@since			0.9.0
			 */
			void add(ccDrawable *dr);
			
			/**
			 *	Increment the selector.
			 *	@since			0.9.0
			 */
			void incrementSelected();
			
			/**
			 *	Decrement the selector.
			 *	Subtracts one from the selected item, unless it is 0.
			 *	@since			0.9.0
			 */
			void decrementSelected();
			
			/**
			 *	Set the selected item.
			 *	@param	sel		The item number to select.
			 *	@since			0.9.0
			 */
			void setSelected(int sel);
			
			/**
			 *	Reset the polygonal selector.
			 *	@param	sel		The new polygonal selector.
			 *	@since			0.9.0
			 */		
			void setSelector(ccMovingPoly *sel);
			
			/**
			 *	Change the Y offset.
			 *	@param	y		The new y offset.
			 *	@since			0.9.0			 
			 */
			void setYOffset(int y);
			
			/**
			 *	Change the X offset.
			 *	@param x		The new x offset.
			 *	@since			0.9.0
			 */
			void setXOffset(int x);
			
			/**
			 *	Change the amount y is changed per item.
			 *	@param	y		The new "delta y".
			 *	@since			0.9.0
			 */
			void setDeltaY(int y);
			
			/**
			 *	Change the amount x is changed per item.
			 *	@param	x		The new "delta x".
			 *	@since			0.9.0
			 */
			void setDeltaX(int x);
			
			/**
			 *	Change the event handler.
			 *	@param	eh		The new event handler, takes two arguments
			 *					representing the selected item and a user
			 *					specified parameter.
			 *	@since			0.9.0
			 */
			void setEventHandler(void (*eh)(int, void *));
	};
}

#endif
