/*	ccpolygon.h
	Polygons (drawable)
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccpolygon.h
 *	@brief		Polygons.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCPOLYGON_H
#define __CCPOLYGON_H

#include <ccsys.h>
#include <cclist.h>
#include <ccvideo.h>
#include <cctexture.h>
#include <ccdrawable.h>

namespace crabclaw	{
	
	/**
	 *	Generic colored polygon class.
	 *	This class provides a simple drawable polygon.
	 *	@since	0.9.0
	 *	@see	ccSceneManager
	 */
	class ccPolygon : public ccDrawable	{
		private:
			int list;
		protected:
			/**
			 *	List of <code>ccVertex</code> objects.
			 *	List of all the verteces that make up this <code>ccPolygon</code>.
			 *	@since			0.9.0
			 */
			ccList<ccVertex> *vert;
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccPolygon</code> for the given primitive list.
			 *	@param	list	The list to create a polygon for.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			ccPolygon(uint8 list);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this <code>ccPolygon</code>.
			 *	@since			0.9.0
			 */
			virtual ~ccPolygon()	{ delete vert; }
			
			/**
			 *	Add a vertex to this <code>ccPolygon</code>.
			 *	Add a vertex to be shown when this <code>ccPolygon</code> is
			 *	drawn.
			 *	@param	v		The <code>ccVertex</code> to add.
			 *	@since			0.9.0
			 */
			void addVertex(ccVertex *v) { vert->add(v); }
			
			/**
			 *	Draw this <code>ccPolygon</code>.
			 *	Draw this <code>ccPolygon</code> to the given
			 *	<code>ccScreen</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			virtual void draw(ccScreen *scr);
			
			/**
			 *	Get the primitive list this <code>ccPolygon</code> belongs in.
			 *	@return			Constant representing the primitive list.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			int getList() { return list; }
			
			/**
			 *	Get an arbitrary vertex from this <code>ccPolygon</code>.
			 *	Get a vertex from the list of verteces.
			 *	@param	num		The <code>ccVertex</code> to get.
			 *	@return			The <code>ccVertex</code>.
			 *	@since			0.9.0
			 */
			ccVertex *getVertex(int num)	{ return vert->get(num); }
	};
	
	/**
	 *	Moving colored polygon class.
	 *	This class provides a moving drawable polygon.
	 *	@since	0.9.0
	 *	@see	ccSceneManager
	 */
	class ccMovingPoly : public ccPolygon	{
		private:
			void (*mv)(ccMovingPoly&);
		public:
			/**
			 *	Constructor.
			 *	Create a new <code>ccMovingPoly</code>.
			 *	@param	list	The primitive list it will belong to.
			 *	@param	mvfunc	A callback called each frame to allow the
			 *					code to move this <code>ccMovingPoly</code>.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 *	@see			setVertex()
			 */
			ccMovingPoly(uint8 list, void (*mvfunc)(ccMovingPoly&));
			
			/**
			 *	Destructor.
			 *	Cleans up memory associated with this <code>ccMovingPoly</code>.
			 *	@since			0.9.0
			 */
			~ccMovingPoly()	{ }
			
			/**
			 *	Draw this <code>ccMovingPoly</code>.
			 *	Draw this <code>ccMovingPoly</code> to the given
			 *	<code>ccScreen</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			void draw(ccScreen *scr);
			
			/**
			 *	Set an arbitrary vertex.
			 *	Set a specified vertex to another.
			 *	@param	num		The vertex number to modify.
			 *	@param	v		The new vertex.
			 *	@since			0.9.0
			 */
			void setVertex(int num, ccVertex *v);
			
			/**
			 *	Move this <code>ccMovingPoly</code>.
			 *	Change all vertecies by a specified amount.
			 *	@param	x		The amount to change x by.
			 *	@param	y		The amount to change y by.
			 *	@param	z		The amount to change z by.
			 *	@since			0.9.0
			 */
			 void move(int x, int y, int z);
	};
			
	/**
	 *	Generic textured polygon class.
	 *	This class provides a textured drawable polygon.
	 *	@since	0.9.0
	 *	@see	ccSceneManager
	 */
	class ccTexturedPoly : public ccPolygon	{
		private:
			ccTexture *txr;
		public:
			/**
			 *	Constructor.
			 *	Create a new <code>ccTexturedPoly</code>.
			 *	@param	list	The primitive list it will belong to.
			 *	@param	txrptr	The texture of this polygon.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			ccTexturedPoly(int list, ccTexture *txrptr);
			
			/**
			 *	Destructor.
			 *	Cleans up memory associated with this <code>ccTexturedPoly</code>.
			 *	@since			0.9.0
			 */
			~ccTexturedPoly() {	}
			
			/**
			 *	Draw this <code>ccTexturedPoly</code>.
			 *	Draw this <code>ccTexturedPoly</code> to the given
			 *	<code>ccScreen</code>.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			void draw(ccScreen *scr);
	};
}

#endif
