/*	ccscenemgr.h
	Scene manager
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccscenemgr.h
 *	@brief		<code>ccSceneManager</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCSCENEMGR_H
#define __CCSCENEMGR_H

#include <ccsys.h>
#include <ccvideo.h>
#include <cclist.h>
#include <ccdrawable.h>
#include <time.h>

namespace crabclaw	{
	/**
	 *	Scene-management class.
	 *	This class provides a mechanism to automate the drawing of
	 *	scenes.
	 *	@since	0.9.0
	 *	@see	ccDrawable
	 */
	class ccSceneManager	{
		private:
			ccList<ccDrawable> *op_list;
			ccList<ccDrawable> *tr_list;
			ccScreen * screen;
			time_t sttime;
			uint32 fcount;
			void (*callback)(ccSceneManager*);
			bool enabled;
			
		public:
			/**
			 *	Constructor.
			 *	@param	func	A callback function called every frame.
			 *	@param	scr		The <code>ccScreen</code> object to output to.
			 *	@since			0.9.0
			 */
			ccSceneManager(void (*func)(ccSceneManager*), ccScreen * scr);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this <code>ccSceneManager</code>
			 *	@since			0.9.0
			 */
			~ccSceneManager();
			
			/**
			 *	Add a new <code>ccDrawable</code> to the scene.
			 *	Determine the list used by the object passed, and place
			 *	that object in the appropriate list.
			 *	@param	obj		The <code>ccDrawable</code> object to add.
			 *	@since			0.9.0
			 */
			void add(ccDrawable *obj);
			
			/**
			 *	Remove a <code>ccDrawable</code> from the scene.
			 *	Determine the list, and remove the object from that list,
			 *	if the object is in the list.
			 *	@param	obj		The <code>ccDrawable</code> to remove.
			 *	@since			0.9.0
			 */
			void remove(ccDrawable *obj);
			
			/**
			 *	Remove a <code>ccDrawable</code> from the scene.
			 *	Based on the list type, and the number of the object,
			 *	remove it from the drawing list.
			 *	@param	list	The list type.
			 *	@param	num		The object's number.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			void remove(uint8 list, int num);
		
			/**
			 *	Get the frames-per-second count.
			 *	Get the average frames-per-second since this
			 *	<code>ccSceneManager</code> was created.
			 *	@return			The FPS count.
			 *	@since			0.9.0
			 */
			float getFPS();
			
			/**
			 *	Start this <code>ccSceneManager</code>.
			 *	Start up this <code>ccSceneManager</code>'s control of
			 *	the display. This function will not return until 
			 *	<code>stop()</code> is called on this object.
			 *	@since			0.9.0
			 */
			void run();
			
			/**
			 *	Stop this <code>ccSceneManager</code>.
			 *	Break this <code>ccSceneManager</code>'s control of
			 *	the display.
			 *	@since			0.9.0
			 */
			void stop();		
	};
}

#endif
