/*	ccsprite.h
	Sprites (drawable)
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccsprite.h
 *	@brief		Declaration of the <code>ccSprite</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCSPRITE_H
#define __CCSPRITE_H

#include <ccsys.h>
#include <cclist.h>
#include <ccvideo.h>
#include <cctexture.h>
#include <ccdrawable.h>

namespace crabclaw	{
	/**
	 *	Drawable sprites.
	 *	This class is designed to be used in conjunction with
	 *	the <code>ccSceneManager</code> class to facilitate the drawing of
	 *	sprites in a scene.
	 *	@see	ccSceneManager
	 *	@see	ccDrawable
	 *	@since	0.9.0
	 */
	class ccSprite : public ccDrawable	{
		private:
			int list;
			float uf;
			float vf;
			int sp_x;
			int sp_y;
			int xpos;
			int ypos;
			int zpos;
			uint32 argb;
			uint32 oargb;
			int px;
			int py;
			int dw;
			int dh;
			ccTexture *txr;
			
		public:
			/**
			 *	Constructor.
			 *	Creates a new <code>ccSprite</code> from the specified
			 *	<code>ccTexture</code>.
			 *	@param	tex		The texture to be used by the sprite.
			 *	@param	u		The percent of the horizontal size of
			 *					the <code>ccTexture</code> that each
			 *					sprite frame occupies.
			 *	@param v		The percent of the vertical size of
			 *					the <code>ccTexture</code> that each
			 *					sprite frame occupies.
			 *	@param listtype	The list that this <code>ccSprite</code>
			 *					should be drawn in.
			 *	@since			0.9.0
			 */
			ccSprite(ccTexture *tex, float u, float v, uint8 listtype);
			
			/**
			 *	Destructor.
			 *	Frees memory used by this <code>ccSprite</code> but
			 *	does not free the <code>ccTexture</code> used by it.
			 */
			~ccSprite();
			
			/**
			 *	Draw this <code>ccSprite</code>.
			 *	Draw this <code>ccSprite</code> to the specified
			 *	<code>ccScreen</code> object.
			 *	@param	scr		The <code>ccScreen</code> to draw to.
			 *	@since			0.9.0
			 */
			void draw(ccScreen *scr);
			
			/**
			 *	Get the primitive list of this <code>ccSprite</code>.
			 *	Get the primitive list of this <code>ccSprite</code>,
			 *	to identify what a <code>ccSceneManager</code> object
			 *	should place this <code>ccSprite</code> in.
			 *	@return			An integer representing a primitive list.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			int getList() { return list; }
			
			/**
			 *	Set the screen position of this <code>ccSprite</code>.
			 *	Set the position that this <code>ccSprite</code> is
			 *	displayed on the <code>ccScreen</code> to which it is 
			 *	drawn.
			 *	@param	x		The horizontal position to display at.
			 *	@param	y		The vertical position to display at.
			 *	@param	z		The depth to display at.
			 *	@since			0.9.0
			 */
			void setPos(int x, int y, int z);
			
			/**
			 *	Set the color to shade this <code>ccSprite</code>.
			 *	On implementations that support color calculation,
			 *	set the color to shade this <code>ccSprite</code>.
			 *	@param	a		The ARGB value to shade.
			 *	@param	o		The OARGB value to offset.
			 *	@since			0.9.0
			 */
			void setColors(uint32 a, uint32 o = 0);
			
			/**
			 *	Set the displayed size of this <code>ccSprite</code>.
			 *	Set the size which this <code>ccSprite</code> is to
			 *	drawn on the <code>ccScreen</code> with.
			 *	@param	w		The desired width, in pixels.
			 *	@param	h		The desired height, in pixels.
			 *	@since			0.9.0
			 */
			void setDimensions(int w, int h);
			
			/**
			 *	Set the displayed frame of this <code>ccSprite</code>.
			 *	Set which frame of this <code>ccSprite</code> will be
			 *	drawn the next time that <code>draw()</code> is called.
			 *	@param	x		The horizontal frame.
			 *	@param	y		The vertical frame.
			 *	@since			0.9.0
			 */
			void setDisplayed(int x, int y);
	};
}

#endif
