/*	cctexture.h
	Texture class
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		cctexture.h
 *	@brief		<code>ccTexture</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
/*	All the private variables in this class are platform-dependent, and
	the implementation of all (non loading) methods should also be as such. However
	all implementations should share the same public methods.	*/
	
#ifndef __CCTEXTURE_H
#define __CCTEXTURE_H

#include <ccsys.h>
#include <ccvideo.h>

namespace crabclaw	{
	
	/**
	 *	Texture class.
	 *	This class wraps around the functionality provided by
	 *	the video hardware to provide a common texturing interface.
	 *	@since	0.9.0
	 */	
	class ccTexture	{
		private:
	
#ifdef _arch_dreamcast
			pvr_poly_hdr_t hdr_op;
			pvr_poly_hdr_t hdr_tr;
			pvr_ptr_t ptr;
#endif
			int w;
			int h;
	
		public:
			/**
			 *	Constant representing a RGB565 texture.
			 *	@since			0.9.0
			 */
			static const uint32 TXRFMT_RGB565 = (1 << 27);
			
			/**
			 *	Constant representing an ARGB4444 texture.
			 *	@since			0.9.0
			 */
			static const uint32 TXRFMT_ARGB4444 = (2 << 27);
			
			/**
			 *	Constant representing an ARGB1555 texture.
			 *	@since			0.9.0
			 */
			static const uint32 TXRFMT_ARGB1555 = 0;
			
			/**
			 *	Constructor.
			 *	System-dependent method to create a texture.
			 *	@param	txr		Texture data.
			 *	@param	w		Width of the texture.
			 *	@param	h		Height of the texture.
			 *	@param	fmt		Pixel-format of the texture.
			 *	@since			0.9.0
			 *	@see			TXRFMT_RGB565
			 *	@see			TXRFMT_ARGB4444
			 *	@see			TXRFMT_ARGB1555
			 */
			ccTexture(void *txr, int w, int h, int fmt);
			
			/**
			 *	Destructor.
			 *	System-dependent method to destroy a texture.
			 *	@since			0.9.0
			 */
			~ccTexture();
			
			/**
			 *	Create a texture from a PNG image.
			 *	Parse a PNG graphic with 
			 *	<a href="http://www.libpng.org/">libpng</a> and call
			 *	the <code>ccTexture</code> constructor to handle
			 *	creating an actual texture.
			 *	@param	fn		Filename of the PNG image.
			 *	@return			A new <code>ccTexture</code>.
			 *	@since			0.9.0
			 */
			static ccTexture *createFromPNG(char *fn);
			
			/**
			 *	Submit texture data to video hardware.
			 *	Platform-dependent method to set video hardware up
			 *	to draw a polygon with this <code>ccTexture</code>.
			 *	You MUST call <code>ccScreen::finishPoly()</code>
			 *	when you are done submiting vertex data for the textured
			 *	polygon.
			 *	@param	scr		The <code>ccScreen</code> to submit to.
			 *	@since			0.9.0
			 *	@see			ccScreen::beginPoly()
			 */
			void submit(ccScreen *scr);
			
			/**
			 *	Get the width of the texture.
			 *	Get this <code>ccTexture</code>'s width in pixels.
			 *	@return			Width, in pixels.
			 */			 
			int getWidth();
			
			/**
			 *	Get the height of the texture.
			 *	Get this <code>ccTexture</code>'s height in pixels.
			 *	@return			Height, in pixels.
			 */
			int getHeight();
	};
}

#endif
