/*	ccvertex.h
	Vertex class
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccvertex.h
 *	@brief		<code>ccVertex</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCVERTEX_H
#define __CCVERTEX_H
	
#include <ccsys.h>

/* Yes, this vertex class was influenced by the pvr_vertex_t type of KallistiOS */

namespace crabclaw	{
	
	/**
	 *	Vertex class.
	 *	This class wraps around the functionality provided by
	 *	the video hardware to provide a common vertex type.
	 *	@since	0.9.0
	 */	
	class ccVertex	{
		private:
			uint32 flags;
			uint16 x;
			uint16 y;
			uint16 z;
			float u;
			float v;
			uint32 argb;
			uint32 oargb;
			
		public:
			/**
			 *	Constant representing a normal vertex.
			 *	@since			0.9.0
			 */
			static const uint32 NORMAL_VERT = 0xe0000000;
			/**
			 *	Constant representing the final vertex of a polygon.
			 *	@since			0.9.0
			 */
			static const uint32 END_VERT = 0xf0000000;
			
			
			/**
			 *	Constructor.
			 *	Creates a new vertex based on the passed parameters.
			 *	@param	f		Vertex flags (type of vertex).
			 *	@param	xp		X position of this <code>ccVertex</code>.
			 *	@param	yp		Y position of this <code>ccVertex</code>.
			 *	@param	zp		Z position of this <code>ccVertex</code>.
			 *	@param	uval	For texturing, the U coordinate of the texture.
			 *	@param	vval	For texturing, the V coordinate of the texture.
			 *	@param	argbcol	The color of this vertex.
			 *	@param	oargbcl	The offset color.
			 *	@since			0.9.0
			 */
			ccVertex(uint32 f, uint16 xp, uint16 yp, uint16 zp, 
			         float uval = 0.0f, float vval = 0.0f, 
			         uint32 argbcol = 0xFFFFFFFF, uint32 oargbcl = 0)	{
				flags = f;
				x = xp;
				y = yp;
				z = zp;
				u = uval;
				v = vval;
				argb = argbcol;
				oargb = oargbcl;
			}
			
			/**
			 *	Destructor.
			 *	Cleans up any memory associated with this <code>ccVertex</code>
			 *	@since			0.9.0
			 */			
			~ccVertex()		{	}
			
			/**
			 *	Get this <code>ccVertex</code>'s flags.
			 *	@return			Vertex flags.
			 *	@since			0.9.0
			 */
			uint32 getFlags()	const	{	return flags;	}
			
			/**
			 *	Get this <code>ccVertex</code>'s X coordinate.
			 *	@return			X coordinate.
			 *	@since			0.9.0
			 */
			uint16 getX()		const	{	return x;		}
			
			/**
			 *	Get this <code>ccVertex</code>'s Y coordinate.
			 *	@return			Y coordinate.
			 *	@since			0.9.0
			 */
			uint16 getY()		const	{	return y;		}
			
			/**
			 *	Get this <code>ccVertex</code>'s Z coordinate.
			 *	@return			Z coordinate.
			 *	@since			0.9.0
			 */
			uint16 getZ()		const	{	return z;		}
			
			/**
			 *	Get this <code>ccVertex</code>'s U coordinate.
			 *	@return			U coordinate.
			 *	@since			0.9.0
			 */
			float getU()		const	{	return u;		}
			
			/**
			 *	Get this <code>ccVertex</code>'s V coordinate.
			 *	@return			V coordinate.
			 *	@since			0.9.0
			 */
			float getV() 		const	{	return v;		}
			
			/**
			 *	Get this <code>ccVertex</code>'s ARGB color.
			 *	@return			ARGB8888 color.
			 *	@since			0.9.0
			 */
			uint32 getARGB() 	const	{	return argb;	}
			
			/**
			 *	Get this <code>ccVertex</code>'s OARGB color.
			 *	@return			OARGB color.
			 *	@since			0.9.0
			 */
			uint32 getOARGB() 	const	{	return oargb;	}
			
			/**
			 *	Set this <code>ccVertex</code>'s X coordinate.
			 *	@param	nx		New X coordinate.
			 *	@since			0.9.0
			 */
			void setX(int nx)			{	x = nx;			}
			
			/**
			 *	Set this <code>ccVertex</code>'s Y coordinate.
			 *	@param	ny		New Y coordinate.
			 *	@since			0.9.0
			 */
			void setY(int ny)			{	y = ny;			}
			
			/**
			 *	Set this <code>ccVertex</code>'s Z coordinate.
			 *	@param	nz		New Z coordinate.
			 *	@since			0.9.0
			 */
			void setZ(int nz)			{	z = nz;			}
			
	};		
}

#endif	
