/*	ccvideo.h
	Screen class...
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccvideo.h
 *	@brief		<code>ccScreen</code> class definition.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
	
#ifndef __CCVIDEO_H
#define __CCVIDEO_H

#include <ccsys.h>
#include <ccvertex.h>

namespace crabclaw	{
	/**
	 *	A screen.
	 *	This class represents a screen in libcrabclaw
	 *	@since	0.9.0
	 */
	class ccScreen	{
		private:
			int x;
			int y;
			int w;
			int h;
			int bpp;
			char *title;
			uint8 open_list;
			
			void initScreen();
			void destroyScreen();
			
		public:
			/**
			 *	Constant representing the opaque primitive list.
			 *	@since			0.9.0
			 */
			static const uint8 LIST_OPAQUE = 0;
			/**
			 *	Constant representing the translucent primitive list.
			 *	@since			0.9.0
			 */
			static const uint8 LIST_TRANSLUCENT = 2;
		
			/**
			 *	Constructor.
			 *	Creates a screen, and initializes the video hardware.
			 *	@param	xpos	Horizontal position of the window.
			 *	@param	ypos	Vertical position of the window.
			 *	@param	wid		Width of the display.
			 *	@param	hei		Height of the display.
			 *	@param	bp		Pixel depth, in bits-per-pixel.
			 *	@param	t		Title of the window created.
			 *	@since			0.9.0
			 */
			ccScreen(uint16 xpos, uint16 ypos, uint16 wid, uint16 hei, uint16 bp, char *t);
			
			/**
			 *	Constructor.
			 *	Creates a screen, and initializes the video hardware.
			 *	This constructor makes a 16bpp display.
			 *	@param	wid		Width of the display.
			 *	@param	hei		Height of the display.
			 *	@param	t		Title of the window created.
			 *	@since			0.9.0
			 */
			ccScreen(uint16 wid, uint16 hei, char *t);
			
			/**
			 *	Constructor.
			 *	Creates a screen, and initializes the video hardware.
			 *	This constructor makes a 16bpp display.
			 *	@param	wid		Width of the display.
			 *	@param	hei		Height of the display.
			 *	@since			0.9.0
			 */
			ccScreen(uint16 wid, uint16 hei);
			
			/**
			 *	Constructor.
			 *	Creates a screen, and initializes the video hardware.
			 *	This constructor makes a 640x480x16 display.
			 *	@param	t		Title of the window created.
			 *	@since			0.9.0
			 */
			ccScreen(char *t);
			
			/**
			 *	Destructor.
			 *	Cleans up any memory used by this <code>ccScreen</code>.
			 *	@since			0.9.0
			 */
			~ccScreen();
			
			/**
			 *	Begin collecting data for a new frame of video output.
			 *	Set up the video hardware to start a new frame of video.
			 *	@since			0.9.0
			 */
			void beginScene();
			
			/**
			 *	Finish collecting data for a frame of video output.
			 *	Inform the video hardware that we are done, and to
			 *	swap framebuffers in a multi-buffered environment.
			 *	@since			0.9.0
			 */
			void finishScene();
			
			/**
			 *	Begin collecting data for a given type of primitive.
			 *	Sets up the video hardware for processing a given
			 *	type of primitive. All primitives submitted between
			 *	the time that <code>beginList()</code> is called and 
			 *	when <code>finishList()</code> is called MUST be of
			 *	that type, or else, as Dan Potter put it (in
			 *	<a href="http://gamedev.allusion.net/" target="_new">
			 *	KallistiOS</a>) "submitting a primitive of the wrong 
			 *	type will quite likely ruin your scene."
			 *	@param	list	The primitive list to begin.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			void beginList(uint8 list);
			
			/**
			 *	Finish collecting data for the open primitive list.
			 *	Close up the open primitive list. DO NOT submit
			 *	primitives after calling <code>finishList()</code>
			 *	as the behavior is undefined, and could cause bad
			 *	things to happen.
			 *	@since			0.9.0
			 */
			void finishList();
			
			/**
			 *	Begin collecting vertex data for a polygon.
			 *	Begin collecting vertex data for a polygon of the
			 *	current list type. This assumes a colored, non-textured
			 *	polygon.
			 *	@since			0.9.0
			 *	@see			ccPolygon
			 *	@see			ccTexture::submit()
			 */
			void beginPoly();
			
			/**
			 *	Finish collecting data for a polygon.
			 *	Close out the current polygon. DO NOT submit any
			 *	verteces after calling <code>finishPoly()</code>,
			 *	as it could cause scene-corruption.
			 *	@since			0.9.0
			 */
			void finishPoly();
			
			/**
			 *	Submit a vertex to video hardware.
			 *	Submits a vertex for the currently opened polygon
			 *	to the video hardware.
			 *	@since			0.9.0
			 */
			void submitVertex(ccVertex v);
			
			/**
			 *	Get the open primitive list.
			 *	Get the currently open primitive list, if any.
			 *	@returns		A constant representing a list type.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			uint8 getOpenList()		const	{	return open_list;	}
	};
}

#endif
