/*	ccmap.cc
	.ljm map handling
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccmap.h>

namespace crabclaw	{
	ccLJMMap::ccLJMMap(const char *fn, int scale)	{
		unsigned long i;
		FILE * fp;
		char * tmp;
		uint64 sttime;
		char sig[4];
	
		fp = fopen(fn, "rb");
		if(fp == NULL)	{
			fprintf(stderr, "ljm_load: Cannot open file: %s\n", fn);
			return;
		}
	
		fread(sig, 3, 1, fp);
		if(strncmp(sig, "LJM", 3))	{
			fprintf(stderr, "ljm_load: File %s is not a valid ljm map.\n", fn);
			return;
		}
	
		fread(&version, sizeof(uint32), 1, fp);
		fread(&dim, sizeof(uint8), 1, fp);
		fread(&tcount, sizeof(uint32), 1, fp);
		fread(&map_x, sizeof(uint32), 1, fp);
		fread(&map_y, sizeof(uint32), 1, fp);
	
		if(version != 1)	{
			fread(&ecount, sizeof(uint32), 1, fp);
			fread(&acount, sizeof(uint32), 1, fp);
		}
	
		tile = (ljmTile *) malloc(sizeof(ljmTile) * tcount);
		fread(tile, sizeof(ljmTile), tcount, fp);
	
		if(version != 1)	{
			event = (ljmEvent *) malloc(sizeof(ljmEvent) * ecount);
			anim = (ljmAnim *) malloc(sizeof(ljmAnim) * acount);
			fread(event, sizeof(ljmEvent), ecount, fp);
			for(i = 0; i < acount; i++)	{
				fread(&anim[i].count, sizeof(uint32), 1, fp);
				anim[i].seq = (uint32 *) malloc(sizeof(uint32) * anim[i].count);
				fread(&anim[i].seq, sizeof(uint32), anim[i].count, fp);
				fread(&anim[i].delay, sizeof(uint8), 1, fp);
			}
		}
		
		tiles_txr = (ccTexture **) malloc(sizeof(ccTexture *) * tcount);
			
		tmp = (char *) malloc(dim * dim * 2);
	
		for(i = 0; i < tcount; i++)	{
			fread(tmp, dim * dim * 2, 1, fp);
			tiles_txr[i] = new ccTexture(tmp, dim, dim, ccTexture::TXRFMT_RGB565);
		}
	
		free(tmp);
	
		tile_map = (uint32 **) malloc(sizeof(uint32 *) * map_y);
		for(i = 0; i < map_y; i++)	{
			tile_map[i] = (uint32 *) malloc(sizeof(uint32) * map_x);
			fread(tile_map[i], sizeof(uint32), map_x, fp);
		}
	
		fclose(fp);
	
		if(version != 1)	{
			sttime = timer_ms_gettime64();
			for(i = 0; i < acount; i++)	{
				anim[i].sttime = sttime;
				anim[i].current = 0;
			}
		}
	
		xoff = yoff = 0;
		dim *= scale;
	}
	
	ccLJMMap::~ccLJMMap()	{	
		uint32 i;
	
		for(i = 0; i < tcount; i++)	{
			delete tiles_txr[i];
		}
		
		free(tiles_txr);
	
		free(tile);
	
		for(i = 0; i < map_y; i++)	{
			free(tile_map[i]);
		}
	
		free(tile_map);
	
		if(version != 1)	{
			for(i = 0; i < acount; i++)	{
				free(anim[i].seq);
			}
			free(anim);
			free(event);
		}
	}
	
	ljmTile *ccLJMMap::getBlock(uint32 blk)	{
		if(blk < 2147483649UL)	{
			return &tile[blk];
		}
		else	{
			return &tile[anim[blk - 2147483649UL].seq[0]];
		}
	}
	
	ljmTile *ccLJMMap::getBlock(uint32 x, uint32 y)	{
		return getBlock(tile_map[y][x]);
	}
	
	ljmTile *ccLJMMap::getBlockPixels(uint32 x, uint32 y)	{
		return getBlock(tile_map[y / dim][x / dim]);
	}
	
	int ccLJMMap::setXOffset(uint32 off)	{
		if(off > map_x)	{
			fprintf(stderr, "setXOffset: offset %lu is larger than the map's x size", off);
			return -1;
		}

		xoff = off;
		return 0;
	}
	
	int ccLJMMap::setYOffset(uint32 off)	{
		if(off > map_y)	{
			fprintf(stderr, "setYOffset: offset %lu is larger than the map's y size", off);
			return -1;
		}
		
		yoff = off;
		return 0;
	}
	
	int ccLJMMap::setOffsets(uint32 x, uint32 y)	{
		if(setXOffset(x) == 0 && setYOffset(y))	{
			return 0;
		}

		return -1;
	}
	
	int ccLJMMap::center(uint32 x, uint32 y, uint32 scx, uint32 scy)	{
		return setOffsets(x - ((scx / 2) / dim), y - ((scy / 2) / dim));
	}
	
	void ccLJMMap::incrementAnims()	{
		uint64 timer = timer_ms_gettime64();
		uint32 i;
	
		for(i = 0; i < acount; i++)	{
			if(anim[i].sttime + (anim[i].delay * 100) <= timer)	{
				anim[i].sttime = timer;
				if(anim[i].current + 1 == anim[i].count)	{
					anim[i].current = 0;
				}
				else	{
					anim[i].current++;
				}
			}
		}
	}
	
	ljmEvent *ccLJMMap::getEvent(uint32 x, uint32 y)	{
		uint32 i;
		ljmEvent * eventfound = NULL;
	
		for(i = 0; i < ecount; i++)	{
			if(event[i].evx == x && event[i].evy == y)	{
				eventfound = &event[i];
				break;
			}
		}
	
		return eventfound;
	}
	
	ljmEvent *ccLJMMap::getEvent(uint32 x, uint32 y, uint8 type)	{
		ljmEvent *eventfound = getEvent(x, y);
	
		if(eventfound != NULL && eventfound->evtype == type)	{
			return eventfound;
		}
	
		return NULL;
	}
	
	void ccLJMMap::draw(ccScreen *scr)	{
		uint32 i, j, animnum, animcur;
		uint16 x, y;
			
		for(i = 0; i + yoff < map_y; i++)	{
			y = i * dim;
			for(j = 0; j + xoff < map_x ; j++)	{
				x = j * dim;
				if((i + yoff) * dim < 480 && (j + xoff) * dim < 640)	{
					if(tile_map[i + yoff][j + xoff] < 2147483649UL)	{
						/*
						glBindTexture(GL_TEXTURE_2D, map->tiles_txr[map->tile_map[i + map->yoff][j + map->xoff]]);
						glBegin(GL_QUADS);
							glTexCoord2f(0.0f, 0.0f);
							glVertex3f(x, y, 0.0f);
							glTexCoord2f(1.0f, 0.0f);
							glVertex3f(x + map->dim, y, 0.0f);
							glTexCoord2f(1.0f, 1.0f);
							glVertex3f(x + map->dim, y + map->dim, 0.0f);
							glTexCoord2f(0.0f, 1.0f);
							glVertex3f(x, y + map->dim, 0.0f);
						glEnd();
						*/
						tiles_txr[tile_map[i + yoff][j + xoff]]->submit(scr);
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x, y + dim, 1, 0.0f, 1.0f));
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x, y, 1, 0.0f, 0.0f));
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x + dim, y + dim, 1, 1.0f, 1.0f));
						scr->submitVertex(ccVertex(ccVertex::END_VERT, x + dim, y, 1, 1.0f, 0.0f));
						scr->finishPoly();
					}
					else	{
						animnum = tile_map[i + yoff][j + xoff] - 2147483649UL;
						animcur = anim[animnum].current;
						/*
						glBindTexture(GL_TEXTURE_2D, map->tiles_txr[map->anim[animnum].seq[animcur]]);
						glBegin(GL_QUADS);
							glTexCoord2f(0.0f, 0.0f);
							glVertex3f(x, y, 0.0f);
							glTexCoord2f(1.0f, 0.0f);
							glVertex3f(x + map->dim, y, 0.0f);
							glTexCoord2f(1.0f, 1.0f);
							glVertex3f(x + map->dim, y + map->dim, 0.0f);
							glTexCoord2f(0.0f, 1.0f);
							glVertex3f(x, y + map->dim, 0.0f);
						glEnd();
						*/
						tiles_txr[anim[animnum].seq[animcur]]->submit(scr);
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x, y + dim, 1, 0.0f, 1.0f));
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x, y, 1, 0.0f, 0.0f));
						scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x + dim, y + dim, 1, 1.0f, 1.0f));
						scr->submitVertex(ccVertex(ccVertex::END_VERT, x + dim, y, 1, 1.0f, 0.0f));
						scr->finishPoly();
					}
				}
			}
		}
	}
}
