/*	cctexture.cc
	Texture loading functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <cctexture.h>
#include <png.h>

namespace crabclaw	{
	
/* these macros are borrowed from KallistiOS' libpng */
#define LOAD565(r, g, b) (((r>>3)<<11) | ((g>>2)<<5) | ((b>>3)))
#define LOAD4444(r, g, b, a) (((a>>4)<<12)|((r>>4)<<8)|((g>>4)<<4)|((b>>4)))

	ccTexture *ccTexture::createFromPNG(char *fn)	{
		FILE *fp;
		char hdr[9];
		png_structp png_ptr;
		png_infop info_ptr;
		png_infop end_info;
		png_bytepp rowpointers;
		png_uint_32 height, width, rowbytes;
		int bit_depth, color_type, channels, type;
		unsigned int i;
		char *data;
		uint16 *tdata;
	
		fp = fopen(fn, "rb");
		if(!fp)	{
			fprintf(stderr, "cc_load_png: Error opening %s\n", fn);
			fclose(fp);
			return NULL;
		}
	
		fread(hdr, 1, 8, fp);
		if(!png_check_sig((png_bytep) hdr, 8))	{
			printf("not a png\n");
			fclose(fp);
			return NULL;
		}
	
		png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
		if(!png_ptr)	{
			fprintf(stderr, "cc_load_png: Couldn't create PNG structure\n");
			fclose(fp);
			return NULL;
		}
	
		info_ptr = png_create_info_struct(png_ptr);
		if(!info_ptr)	{
			png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
			fprintf(stderr, "cc_load_png: Couldn't create PNG info structure\n");
			fclose(fp);
			return NULL;
		}
	
		end_info = png_create_info_struct(png_ptr);
		if (!end_info)	{
			png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
			fprintf(stderr, "cc_load_png: Couldn't create PNG info structure\n");
			fclose(fp);
			return NULL;
		}
	
		png_init_io(png_ptr, fp);
		png_set_sig_bytes(png_ptr, 8);
		png_read_info(png_ptr, info_ptr);
	
		png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL);
	
		if(color_type == PNG_COLOR_TYPE_PALETTE)
			png_set_palette_to_rgb(png_ptr);
		if(color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
			png_set_gray_1_2_4_to_8(png_ptr);
		if(png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
			png_set_tRNS_to_alpha(png_ptr);
		if(bit_depth == 16)
			png_set_strip_16(png_ptr);
		if(color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
			png_set_gray_to_rgb(png_ptr);
		if(bit_depth < 8)
			png_set_packing(png_ptr);
	
		png_read_update_info(png_ptr, info_ptr);
	
		rowbytes = png_get_rowbytes(png_ptr, info_ptr);
		channels = (int) png_get_channels(png_ptr, info_ptr);
	
		data = (char *) malloc(rowbytes * height);
		if(data == NULL)	{
			png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
			fprintf(stderr, "cc_load_png: Couldn't allocate memory for image\n");
			fclose(fp);
			return NULL;
		}
	
		rowpointers = (png_bytepp )malloc(height * sizeof(png_bytep));
		if(rowpointers == NULL)	{
			png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
			fprintf(stderr, "cc_load_png: Couldn't allocate memory for rowpointers\n");
			free(data);
			fclose(fp);
			return NULL;
		}
	
		for(i = 0; i < height; i++)	{
			rowpointers[i] = (png_bytep) (data + (i * rowbytes));
		}
	
		png_read_image(png_ptr, rowpointers);
	
		png_read_end(png_ptr, end_info);
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
	
		free(rowpointers);
		
		tdata = new uint16[width * height];
	
		if(channels == 4)	{
			type = TXRFMT_ARGB4444;
			for(i = 0; i < width * height; i++)	{
				tdata[i] = LOAD4444(data[i * 4], data[i * 4 + 1], data[i * 4 + 2], data[i * 4 + 3]);
			}
		}
		else	{
			type = TXRFMT_RGB565;
			for(i = 0; i < width * height; i++)	{
				tdata[i] = LOAD565(data[i * 3], data[i * 3 + 1], data[i * 3 + 2]);
			}
		}
		
		free(data);
		
		ccTexture *ret = new ccTexture(tdata, width, height, type);
		
		delete tdata;
		return ret;
	}
	
	int ccTexture::getWidth()	{
		return w;
	}
	
	int ccTexture::getHeight()	{
		return h;
	}
}
